#include "USBDeviceObserver.h"

#include "QUdevEnumerator.h"
#include "QUdev.h"

#include <QDebug>

USBDeviceObserver::USBDeviceObserver(const QUdev &udev, QObject *parent) :
        QUdevMonitor(udev, USBDeviceObserver::Udev, parent),
        m_udev(udev)
{
    connect(this, SIGNAL(deviceAdded(QUdevDevice)), this, SLOT(deviceAdded(QUdevDevice)));
    connect(this, SIGNAL(deviceRemoved(QUdevDevice)), this, SLOT(deviceRemoved(QUdevDevice)));

    if (!addFilterSubsystem(QUdevNS::USB))
        qWarning("USBDeviceObserver: Cannot apply subsystem filter!");

    if (!startListening())
        qWarning("USBDeviceObserver: Failed to start listening!");
}

void USBDeviceObserver::deviceAdded(QUdevDevice device)
{
    QString vendorId = device.value(QUdevNS::VENDOR_ID).toString();
    QString productId = device.value(QUdevNS::MODEL_ID).toString();

    if (vendorId.isEmpty() || productId.isEmpty())
        return;

    qDebug() << "New USB device added:" << device.value(QUdevNS::VENDOR_NAME) << device.value(QUdevNS::MODEL) << ("(" + vendorId + ":" + productId + ").");
    enumerateDevices();
}

void USBDeviceObserver::deviceRemoved(QUdevDevice device)
{
    QString vendorId = device.value("ID_VENDOR_ID").toString();
    QString productId = device.value("ID_MODEL_ID").toString();

    if (vendorId.isEmpty() || productId.isEmpty())
        return;

    qDebug() << "USB device removed:" << device.value(QUdevNS::VENDOR_NAME) << device.value(QUdevNS::MODEL) << ("(" + vendorId + ":" + productId + ").");
    enumerateDevices();
}

void USBDeviceObserver::enumerateDevices()
{
    QUdevEnumerator e(m_udev, this);
    e.addFilterProperty(QUdevNS::VENDOR_ID);
    e.addFilterSubsystem(QUdevNS::USB);

    QUdevDeviceList devices = e.listDevices();
    qDebug() << "Currently there are" << devices.count() << "recognised devices.";
    int i = 1;
    foreach (const QUdevDevice &d, devices)
    {
        qDebug() << "Device no" << i;
        qDebug() << "  Vendor:" << d.value(QUdevNS::VENDOR_NAME).toString();
        qDebug() << "  Model: " << d.value(QUdevNS::MODEL).toString();
        qDebug() << "  ID:    " << (d.value(QUdevNS::VENDOR_ID).toString() + ":" + d.value(QUdevNS::MODEL_ID).toString());
        qDebug() << "------------";

        i++;
    }
}
