#ifndef QUDEV_P_H
#define QUDEV_P_H

#include <QAtomicInt>
#include <QHash>
#include <QString>
#include <QStringList>
#include <QSharedData>

#include <libudev.h>

#include "QUdevDevice.h"


class QUdevContext
{
public:
    QUdevContext();
    QUdevContext(const QUdevContext &other);
    ~QUdevContext();

    struct udev *coreContext() const;

private:
    QUdevContext &operator=(const QUdevContext &other);
    struct udev *m_coreContext;
};


class QUdevData : public QSharedData {
public:
    QUdevData();
    QUdevData(const QUdevData &other);
    ~QUdevData();

    const QUdevContext &context() const;
    struct udev *coreContext() const;

private:
    QUdevContext m_context;
};

namespace QUdevNS {
inline void fillQStringList(QStringList &list, struct udev_list_entry *firstEntry)
{
    struct udev_list_entry *entry = firstEntry;

    while (entry)
    {
        QString name = QLatin1String(udev_list_entry_get_name(entry));
        list.append(name);

        entry = udev_list_entry_get_next(entry);
    }
}

inline QString propertyToString(QUdevNS::Property p)
{
    switch (p)
    {
    case QUdevNS::VENDOR_ID:
        return QLatin1String("ID_VENDOR_ID");
    case QUdevNS::MODEL_ID:
        return QLatin1String("ID_MODEL_ID");
    case QUdevNS::VENDOR_NAME:
        return QLatin1String("ID_VENDOR_ENC");
    case QUdevNS::BUS:
        return QLatin1String("ID_BUS");
    case QUdevNS::DEVPATH:
        return QLatin1String("DEVPATH");
    case QUdevNS::MODEL:
        return QLatin1String("ID_MODEL_ENC");
    case QUdevNS::SERIAL:
        return QLatin1String("ID_SERIAL");
    case QUdevNS::DRIVER:
        return QLatin1String("DRIVER");
    case QUdevNS::PRODUCT:
        return QLatin1String("PRODUCT");
    }

    return QLatin1String("AN_UNKNOWN_PROPERTY__");
}

inline QString subsystemToString(QUdevNS::Subsystem s)
{
    switch (s)
    {
    case QUdevNS::INPUT:
        return QLatin1String("input");
    case QUdevNS::USB:
        return QLatin1String("usb");
    }

    return QLatin1String("AN_UNKNOWN_SUBSYSTEM__");
}

} // namespace QUdev

#ifndef L1STR // Only null strings will be represented as a NULL pointer.
#define L1STR(string) (string.isNull() ? NULL : string.toLatin1().constData())
#endif
#ifndef L1STR2 // Null AND empty strings will be represented as a NULL pointer.
#define L1STR2(string) (string.isEmpty() ? NULL : string.toLatin1().constData())
#endif

#endif // QUDEV_P_H
