#ifndef QUDEVCONTEXTTEST_H
#define QUDEVCONTEXTTEST_H

#include <cppunit/TestAssert.h>
#include <cppunit/TestCase.h>
#include <cppunit/TestSuite.h>
#include <cppunit/TestCaller.h>
#include "QUdev_p.h"
#include "QUdev.h"
#include "QUdevMonitor.h"

class QUdevContextTest : public CppUnit::TestCase
{
public:
    QUdevContextTest() {}

    void testCoreContext()
    {
        CPPUNIT_ASSERT_MESSAGE("Core context should not be NULL!", m_context.coreContext() != NULL);
    }

private:
    QUdevContext m_context;
};

class QUdevPrivateTest : public CppUnit::TestFixture
{
public:
    QUdevPrivateTest() {}

    void setUp()
    {
        m_private = new QUdevPrivate();
    }

    void tearDown()
    {
        delete m_private;
    }

    void testReferencing()
    {
        CPPUNIT_ASSERT_EQUAL(1, int(m_private->ref));
        QUdevPrivate *referenced = m_private->referenced();
        CPPUNIT_ASSERT_EQUAL(m_private, referenced);
        CPPUNIT_ASSERT_EQUAL(2, int(referenced->ref));
    }

private:
    QUdevPrivate *m_private;
};

class QUdevTest : public CppUnit::TestFixture
{
public:
    QUdevTest() {}

    void setUp()
    {
        m_u = new QUdev();
    }

    void tearDown()
    {
        delete m_u;
    }

    void testCopyConstructor()
    {
        QUdev u2(*m_u);
        QUdev u3;
//        CPPUNIT_ASSERT((*m_u) == u2);
        CPPUNIT_ASSERT(m_u->context().coreContext() == u2.context().coreContext());
//        CPPUNIT_ASSERT(*m_u != u3);
        CPPUNIT_ASSERT(m_u->context().coreContext() != u3.context().coreContext());
    }

    void testSysPath()
    {
        CPPUNIT_ASSERT(!m_u->sysPath().isEmpty());
    }

    void testDevPath()
    {
        CPPUNIT_ASSERT(!m_u->devPath().isEmpty());
    }

    static CppUnit::Test *suite()
    {
        CppUnit::TestSuite *suite = new CppUnit::TestSuite("QUdevTest");
        suite->addTest(new CppUnit::TestCaller<QUdevTest>("testCopyConstructor", &QUdevTest::testCopyConstructor));
        suite->addTest(new CppUnit::TestCaller<QUdevTest>("testSysPath", &QUdevTest::testSysPath));
        suite->addTest(new CppUnit::TestCaller<QUdevTest>("testDevPath", &QUdevTest::testDevPath));
        return suite;
    }

private:
    QUdev *m_u;
};

class QUdevMonitorTest : public CppUnit::TestFixture
{
public:
    QUdevMonitorTest() {}

    void setUp()
    {
        m_monitor = new QUdevMonitor(m_udev);
    }

    void tearDown()
    {
        delete m_monitor;
    }

    void testListening()
    {
        CPPUNIT_ASSERT_EQUAL(false, m_monitor->isListening());
        CPPUNIT_ASSERT_EQUAL(true, m_monitor->startListening());
        CPPUNIT_ASSERT_EQUAL(true, m_monitor->isListening());
        CPPUNIT_ASSERT_EQUAL(true, m_monitor->stopListening());
        CPPUNIT_ASSERT_EQUAL(false, m_monitor->isListening());
        CPPUNIT_ASSERT_EQUAL(true, m_monitor->startListening());
        CPPUNIT_ASSERT_EQUAL(true, m_monitor->isListening());
    }

    void testReset()
    {
        CPPUNIT_ASSERT_EQUAL(false, m_monitor->isListening());
        m_monitor->startListening();
        CPPUNIT_ASSERT_EQUAL(true, m_monitor->isListening());
        m_monitor->reset();
        CPPUNIT_ASSERT_EQUAL(false, m_monitor->isListening());
    }

    static CppUnit::Test *suite()
    {
        CppUnit::TestSuite *suite = new CppUnit::TestSuite("QUdevMonitor");
        suite->addTest(new CppUnit::TestCaller<QUdevMonitorTest>("testListening", &QUdevMonitorTest::testListening));
        suite->addTest(new CppUnit::TestCaller<QUdevMonitorTest>("testReset", &QUdevMonitorTest::testReset));
        return suite;
    }

private:
    QUdev m_udev;
    QUdevMonitor *m_monitor;
};

#endif // QUDEVCONTEXTTEST_H
